commands = [
    #RACF commands
    "addgroup",
    "altgroup",
    "adduser", 
    "altuser",
    "addsd",
    "altdsd",
    "deluser",
    "delgroup",
    "deldsd",
    "connect",
    "remove",
    "permit",
    "racmap",
    "raclink",
    "racpriv",
    "racprmck",
    "racdcert",
    "rdefine",
    "ralter",
    "rdelete",
    "rlist",
    "listdsd",
    "listuser",
    "listgrp", 
    "display",
    "search",
    "setropts",
    "target",
    "rvary",

    "password",
    "passphrase",

    #TSO commands
    "PROFILE",
    "ALLOC",
    "EXEC",
    "DELETE",
    "STATUS",
    "LISTALC",
    "LISTBC",
    "LISTCAT",
    "LISTDS",
    "FREE",
    "SMCOPY",
    "OUTPUT",
    "PRINTDS",
    ]